
(defcustom multi-message-timeout 2
  "Number of seconds between messages before clearing the accumulated list."
  :type 'number
  :group 'minibuffer
  :version "28.1")

(defcustom multi-message-max 8
  "Max size of the list of accumulated messages."
  :type 'number
  :group 'minibuffer
  :version "28.1")

(defcustom multi-message-transient "\\.\\.\\.\\'"
  "Regexp to filter out transient messages that should not be stacked."
  :type 'regexp
  :group 'minibuffer
  :version "28.1")

(defvar multi-message-separator "\n")

(defvar multi-message-list nil)

(defun set-multi-message (message)
  "Return recent messages as one string to display in the echo area.
Note that this feature works best only when `resize-mini-windows'
is at its default value `grow-only'."
  (let ((last-message (car multi-message-list)))
    (unless (and last-message (equal message (aref last-message 1)))
      (when last-message
        (cond
         ((> (float-time) (+ (aref last-message 0) multi-message-timeout))
          (setq multi-message-list nil))
         ((or
           ;; `message-log-max' was nil, potential clutter.
           (aref last-message 2)
           (string-match-p multi-message-transient (aref last-message 1)))
          (setq multi-message-list (cdr multi-message-list)))))
      (push (vector (float-time) message (not message-log-max)) multi-message-list)
      (when (> (length multi-message-list) multi-message-max)
        (setf (nthcdr multi-message-max multi-message-list) nil)))
    (mapconcat (lambda (m) (aref m 1))
               (reverse multi-message-list)
               multi-message-separator)))

(setq set-message-function 'set-multi-message)
